from enum import Enum


class UnitType(Enum):
    NANOSECONDS='NANOSECONDS'
    MICROSECONDS='MICROSECONDS'
    MILLISECONDS='MILLISECONDS'
    SECONDS='SECONDS'
    MINUTES='MINUTES'
    HOURS='HOURS'
    DAYS='DAYS'
    WEEKS='WEEKS'